@@ -67,8 +67,9 @@ spl3D <- function(x1,
   EDnom = rep(ncol(B123) - ncol(X), 3)
   ## Remove intercept column to avoid singularity problems.
   X <- removeIntercept(X)
-  # construct list of sparse precision matrices.
-  lGinv <- constructGinvSplines(q, knots, pord)
+  ## Construct list of sparse precision matrices.
+  scaleFactor <- calcScaleFactor(knots, pord)
+  lGinv <- constructGinvSplines(q, knots, pord, scaleFactor)
   names(lGinv) <- paste0("s(", xNames, ")")
   if (is.null(X)) {
     dim.f <- NULL
@@ -83,5 +84,6 @@ spl3D <- function(x1,
   return(list(X = X, Z = B123, lGinv = lGinv, knots = knots,
               dim.f = dim.f, dim.r = dim.r, term.labels.f = term.labels.f,
               term.labels.r = term.labels.r, x = xList,
-              pord = pord, degree = degree, scaleX = scaleX, EDnom = EDnom))
+              pord = pord, degree = degree, scaleX = scaleX, EDnom = EDnom,
+              scaleFactor = scaleFactor))
 }
